@@ -1,4 +1,4 @@
-""""Paper surfer - browse papers posted on the mattermost channel.
+"""Paper surfer - browse papers posted on the mattermost channel.
 
 UI:
 
@@ -10,239 +10,18 @@ UI:
 
 """
 import subprocess
-from dataclasses import dataclass
-import re
 from functools import partial
-import json
 import time
 import os
-import requests
-import mattermostdriver
+import sys
+import re
+from exceptions import ConfigError
 import urwid
 import configargparse
-
-
-class ConfigError(Exception):
-    """Configuration error."""
-
-
-@dataclass
-class PostDTO:
-    """"Encapsulate Mattermost Posts."""
-    id: str
-    message: str
-    reporter: str
-    doi: str
-
-    def __str__(self):
-        return self.message
-
-
-@dataclass
-class PaperDTO:
-    """"Encapsulate Paper meta data."""
-    author: str
-    authors: str
-    title: str
-    journal: str
-    year: int
-    abstract: str
-    doi: str
-    slug: str
-
-
-class Bibtex:
-    def entry_from_doi(self, doi):
-        return Doi().get_bibtex(doi)
-
-    def bib_from_dois(self, dois):
-        return "\n".join([Doi().get_bibtex(doi) for doi in dois])
-
-
-class Doi:
-    """Interface w/ the doi.org api"""
-    def get_doi_link(self, doi):
-        """Assemble doi link."""
-        return f"http://doi.org/{doi}"
-
-    def load_doi_data(self, doi):
-        headers = {
-            'Accept': 'application/json',
-        }
-        return requests.get(f'http://dx.doi.org/{doi}',
-                            headers=headers).content
-
-    def parse_doi_json(self, jsoncontent):
-        """Tranform doi json to PaperDTO"""
-        info = json.loads(jsoncontent)
-
-        with open("debug.json", "w") as file:
-            file.write(json.dumps(info))
-
-        author = (f"{info['author'][0]['given']} {info['author'][0]['family']}"
-                  if "author" in info
-                  else "Author N/A")
-        authors = (", ".join([f"{a['given']} {a['family']}"
-                              for a in info['author']])
-                   if "author" in info
-                   else "Authors N/A")
-        title = (info['title']
-                 if "title" in info
-                    and isinstance(info['title'], str)
-                 else "Title N/A")
-        journal = (info['publisher']
-                   if "publisher" in info
-                   else "Journal N/A")
-        year = info['created']['date-parts'][0][0]
-        doi = info['DOI']
-        abstract = (info['abstract']
-                    if "abstract" in info
-                    else "Abstract N/A")
-
-        slug = f"{info['author'][0]['family']}{year}"
-
-        return PaperDTO(author, authors, title, journal, year, abstract, doi,
-                        slug)
-
-    def get_bibtex(self, doi):
-        headers = {
-            'Accept': 'text/bibliography; style=bibtex',
-        }
-        return requests.get(f'http://dx.doi.org/{doi}', headers=headers).text
-
-    def get_info(self, doi):
-        try:
-            jsoncontent = self.load_doi_data(doi)
-            data = self.parse_doi_json(jsoncontent)
-            return data
-        except json.decoder.JSONDecodeError:
-            return None
-
-    def extract_doi(self, hay):
-        """Parse doi from string, or None if not found.
-
-        >>> Doi().extract_doi("https://doi.org/10.1093/petrology/egaa077")
-        '10.1093/petrology/egaa077'
-        """
-        pattern = r'\b10\.\d{4,9}/[-._;()/:A-Z0-9]+'
-        matches = re.compile(pattern, re.I).search(hay)
-        return matches.group() if matches else None
-
-
-class Mattermost:
-    """Provide a simplified interaction w/ mattermost api."""
-    def __init__(self, url, channelname, username, password):
-        self.mattermost = mattermostdriver.Driver({
-            'url': url,
-            'login_id': username,
-            'password': password,
-            'port': 443
-        })
-
-        try:
-            self.mattermost.login()
-        except (mattermostdriver.exceptions.NoAccessTokenProvided,
-                requests.exceptions.InvalidURL,
-                requests.exceptions.HTTPError):
-            print("Failed to log into Mattermost.")
-            raise ConfigError
-
-        try:
-            self.channel = self.get_channel(channelname)
-        except ConfigError:
-            print("Couldn't find Mattermost channel.")
-            raise ConfigError
-        self.reporters = {}
-
-    def get_channel(self, channelname):
-        """"Try to find the paper channel by display name."""
-        mm = self.mattermost
-        teams = [team["id"] for team in mm.teams.get_user_teams("me")]
-        channels = []
-        for team in teams:
-            teamchannels = [channel for channel
-                            in mm.channels.get_channels_for_user("me", team)
-                            if channel["display_name"] == channelname]
-            channels.extend(teamchannels)
-
-        # lets just hope no-one has the same channel name in multiple teams
-        if len(channels) == 0:
-            print(f"Channel {channelname} does not exits")
-            raise ConfigError
-        return channels[0]["id"]
-
-    def get_reporter(self, id):
-        """Load user from mattermost api and cache."""
-        if id not in self.reporters:
-            self.reporters[id] = self.mattermost.users.get_user(id)["username"]
-
-        return self.reporters[id]
-
-    def retrieve_all_messages(self):
-        """Retrieve all messages from mattermost, unfiltered for papers."""
-        posts = self.mattermost.posts.get_posts_for_channel(self.channel)
-        return [PostDTO(
-                    id=m['id'],
-                    message=m['message'],
-                    reporter=self.get_reporter(m['user_id']),
-                    doi=Doi().extract_doi(m['message']),
-                )
-                for m in posts['posts'].values()]
-
-    def filter_incoming(self, posts):
-        """Filter messages from mattermost to only papers."""
-        return [p for p in posts if "doi" in p.message]
-
-    def retrieve(self):
-        """Retrieve papers from mattermost channel."""
-        msgs = self.retrieve_all_messages()
-        self.msgs = self.filter_incoming(msgs)
-        return self.msgs
-
-    def check_doi_exits(self, doi):
-        doi_needle = Doi().extract_doi(doi)
-        msg_found = [msg for msg in self.msgs
-                     if Doi().extract_doi(msg.doi) == doi_needle]
-        return bool(msg_found)
-
-    def get_filtered(self, needle):
-        """Filter posts by needle."""
-        return [m for m in self.msgs
-                if needle.lower() in m.message.lower()
-                or needle.lower() in m.reporter.lower()]
-
-    def post(self, message):
-        self.mattermost.posts.create_post({"channel_id": self.channel,
-                                           "message": message})
-
-
-class PrettyButton(urwid.WidgetWrap):
-    def __init__(self, label, on_press=None, user_data=None):
-        self.text = urwid.Text("")
-        self.set_label(label)
-        self.widget = urwid.AttrMap(self.text, '', 'highlight')
-
-        # use a hidden button for evt handling
-        self._hidden_btn = urwid.Button(f"hidden {self.label}",
-                                        on_press, user_data)
-
-        super(self.__class__, self).__init__(self.widget)
-
-    def selectable(self):
-        return True
-
-    def keypress(self, *args, **kw):
-        return self._hidden_btn.keypress(*args, **kw)
-
-    def mouse_event(self, *args, **kw):
-        return self._hidden_btn.mouse_event(*args, **kw)
-
-    def get_label(self):
-        return self.label
-
-    def set_label(self, label):
-        self.label = label
-        self.text.set_text(f"[ {label} ]")
+from ui_elements import PrettyButton
+from mattermost import Mattermost
+from doi import Doi
+from bibtex import Bibtex
 
 
 class Papersurfer:
@@ -308,16 +87,19 @@ class Papersurfer:
         self.mainloop.run()
 
     def h_unhandled_input(self, key):
+        """Handle keyboard input not otherwise handled."""
         if key == "esc":
             raise urwid.ExitMainLoop()
 
     def load_list(self, _loop, _data):
+        """Load and display paper list."""
         body = [self.list_item(paper) for paper in self.mtm.retrieve()]
         self.listcontent.clear()
         self.listcontent.extend(body)
         self.mainloop.widget = self.top
 
     def loading_indicator(self):
+        """Create loading indicator dialog."""
         body_text = urwid.Text("Loading...", align='center')
         body_filler = urwid.Filler(body_text, valign='middle')
         body_padding = urwid.Padding(
@@ -329,6 +111,7 @@ class Papersurfer:
         return urwid.Frame(body_padding)
 
     def details_popup(self, paper):
+        """Create Dialog with paper details."""
         header_text = urwid.Text(('banner', 'Paper details'), align='center')
         header = urwid.AttrMap(header_text, 'banner')
 
@@ -391,7 +174,7 @@ class Papersurfer:
         return pile
 
     def updscrn(self):
-        """"Update (redraw) screen."""
+        """Update (redraw) screen."""
         self.mainloop.draw_screen()
 
     def onchange(self, _, needle):
@@ -402,6 +185,7 @@ class Papersurfer:
                                  for paper in self.mtm.get_filtered(needle)])
 
     def running_export(self, state):
+        """Set exporting state."""
         btn = self.exportbutton
         label = btn.get_label()
         running_indicator = " (running...)"
@@ -422,6 +206,7 @@ class Papersurfer:
         self.running_export(False)
 
     def export_to_bibtex(self):
+        """Export current filtered list to bibtex file."""
         papers = self.mtm.get_filtered(self.filter)
         dois = [paper.doi for paper in papers]
         string = Bibtex().bib_from_dois(dois)
@@ -455,16 +240,18 @@ class Papersurfer:
         self.mainloop.widget = self.details_popup(paper)
 
     def h_close_dialog(self, _):
+        """Handle close dialog button."""
         self.close_dialog()
 
     def close_dialog(self):
+        """Close currently open dialog."""
         self.mainloop.widget = self.top
 
     def open_submit_paper(self, _):
-
+        """Open submit paper dialog."""
         self._pile = urwid.Pile(
             [
-                PostDialog(self.mtm, close=self.h_close_dialog)
+                PostDialog(self.mtm, self.h_close_dialog, self.mainloop)
             ]
         )
         self._over = urwid.Overlay(
@@ -494,7 +281,6 @@ def get_config_file_paths():
     >>> type(get_config_file_paths())
     <class 'list'>
     """
-
     env = os.environ
     xdg_home = None
     if 'XDG_CONFIG_HOME' in env:
@@ -517,6 +303,7 @@ def get_config_file_paths():
 
 
 def interactive_configuration():
+    """Query user for credentials."""
     url = input("Mattermost URL (eg. mattermost.example.net): ")
     channel = input("Channel (eg. Paper Club): ")
     username = input("Username (same as mattermost login, "
@@ -526,7 +313,8 @@ def interactive_configuration():
 
 
 class PostDialog(urwid.WidgetWrap):
-    """
+    """Dialog to submit a new paper to mattermost thread.
+
     UI:
         DOI: [ _________________]
         Generated Message:
@@ -534,7 +322,12 @@ class PostDialog(urwid.WidgetWrap):
 
         [Submit]       [Close]
     """
-    def __init__(self, mattermost, close):
+    def __init__(self, mattermost, close, loop):
+        self._loop = loop
+        self.alarm_handle = None
+
+        self.doi = None
+        self.msg = None
         self.mattermost = mattermost
         self.close = close
         self.doi_input = urwid.Edit("Doi: ")
@@ -558,21 +351,21 @@ class PostDialog(urwid.WidgetWrap):
             right=1
         )
         body = urwid.LineBox(body_padding)
-        frame = urwid.Frame(
-                body,
-                header=urwid.Text("Submit new paper to list"),
-            )
+        frame = urwid.Frame(body,
+                            header=urwid.Text("Submit new paper to list"))
 
         self.widget = frame
 
-        super(self.__class__, self).__init__(self.widget)
+        super().__init__(self.widget)
 
     def submit(self, _):
+        """Submit post to thread."""
         if not self.mattermost.check_doi_exits(self.doi):
             self.mattermost.post(self.msg)
         self.close(_)
 
     def create_mgs(self, paper):
+        """Format post message."""
         msg = f"""\
 {paper.title}
 {paper.authors}
@@ -581,10 +374,18 @@ class PostDialog(urwid.WidgetWrap):
         return msg
 
     def h_input(self, _, doi):
-        self.doi_result.set_text("... loading ...")
+        """Handle doi input field and debounce."""
+        self.doi_result.set_text("")
         self.doi = None
         self.msg = None
 
+        self._loop.remove_alarm(self.alarm_handle)
+        self.alarm_handle = self._loop.set_alarm_in(.5, self.search_doi, doi)
+
+    def search_doi(self, _loop, doi):
+        """Trigger search for paper ref by doi and update ui."""
+        self.doi_result.set_text("... loading ...")
+        self._loop.draw_screen()
         if Doi().extract_doi(doi):
             paper = Doi().get_info(doi)
             if paper:
@@ -595,6 +396,8 @@ class PostDialog(urwid.WidgetWrap):
                     self.doi_result.set_text(self.create_mgs(paper))
                     self.doi = doi
                     self.msg = self.create_mgs(paper)
+            else:
+                self.doi_result.set_text("Doi not found.")
             return
 
         self.doi_result.set_text("invalid doi")
@@ -602,8 +405,8 @@ class PostDialog(urwid.WidgetWrap):
 
 def parse_args():
     """Parse command line arguments and config file."""
-    parser = configargparse.ArgParser()
-    parser._default_config_files = get_config_file_paths()
+    parser = configargparse.ArgParser(
+        default_config_files=get_config_file_paths())
     parser.add("-w", "--write-out-config-file",
                help="takes the current command line args and writes them out "
                     "to a config file at the given path",
@@ -630,7 +433,7 @@ def parse_args():
                 Mattermost(url, channel, username, password)
             except ConfigError:
                 print("Failed to validate configuration, exiting.")
-                exit(1)
+                sys.exit(1)
 
             options.url = url
             options.channel = channel
@@ -648,7 +451,7 @@ def parse_args():
             time.sleep(2)
         else:
             parser.print_help()
-            exit(0)
+            sys.exit(0)
 
     return options
 
@@ -661,12 +464,14 @@ def just_papers(url, channel, username, password):
 
 
 def just_bibtex(url, channel, username, password):
+    """Retrieve and dump bibtext formated data, unfiltered."""
     posts = Mattermost(url, channel, username, password).retrieve()
     dois = [post.doi for post in posts]
     print(Bibtex().bib_from_dois(dois))
 
 
 def main():
+    """Run main program."""
     opt = parse_args()
     if opt.dump_posts:
         just_papers(opt.url, opt.channel, opt.username, opt.password)
